'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
var BASE = 10;
var DOT = '.';
var ID = ':';
var ALL = '*';
var AllChildren = '*.';

var splitHistory = {};

var isInString = function isInString(string, toFind) {
  return string.indexOf(toFind) !== -1;
};

function splitString(path) {
  if (splitHistory[path] === undefined) {
    splitHistory[path] = path.split(DOT);
  }

  return splitHistory[path];
}

function getFromJSON(node, path) {
  var parts = splitString(path);
  var ref = node;

  for (var i = 0; i < parts.length; i += 1) {
    if (ref[parts[i]] === undefined) {
      return undefined;
    }

    ref = ref[parts[i]];
  }

  return ref;
}

function findObject(node, path, id) {
  var array = getFromJSON(node, path);
  if (!array) {
    return undefined;
  }

  return array.find(function (element) {
    return element.id === id;
  });
}

function getArrayById(node, key, findNode, read) {
  var path = key.split(ID)[0];
  var suffix = key.split(ID).slice(1).join(ID);

  if (isInString(suffix, DOT)) {
    var _id = parseInt(suffix.split(DOT)[0], BASE);
    var subPath = suffix.replace(/^[0-9]+\./, '');

    var child = findNode(node, path, _id);
    if (!child) {
      console.warn('The path reference by ' + key + ' could not be found.');
      return undefined;
    }
    return read(child, subPath);
  }

  var id = parseInt(suffix, BASE);
  return findNode(node, path, id);
}

var _read = void 0;
function mapObjectChildren(node, path, suffix) {
  var obj = getFromJSON(node, path);
  if (!obj) {
    return undefined;
  }
  return obj.map(function (child) {
    return _read(child, suffix);
  });
}

function getChildren(node, key, mapChildren) {
  var path = key.split(AllChildren)[0];
  var suffix = key.split(AllChildren)[1];

  return mapChildren(node, path, suffix);
}

_read = function read(node, key) {
  if (isInString(key, ID)) {
    return getArrayById(node, key, findObject, _read);
  } else if (isInString(key, ALL)) {
    return getChildren(node, key, mapObjectChildren);
  }

  return getFromJSON(node, key);
};

var has = exports.has = function has(node, key) {
  return _read(node, key) !== undefined;
};
var unwrap = exports.unwrap = function unwrap(node, key) {
  return _read(node, key);
};

exports.default = _read;